@@ -28,7 +28,12 @@
     }
     $edit_cmd.=implode(", ", $parts);
 
-    PFCMD($edit_cmd);
+    # I REALLLLYY mean false (avoids 0, empty strings and empty arrays to pass here)
+    if (PFCMD($edit_cmd) === false) {
+      # an error was shown by PFCMD now die to avoid closing the popup
+      exit();
+    }
+    # no errors from pfcmd, go on
     $edited=true; 
     print "<script type='text/javascript'>opener.focus(); opener.location.href = opener.location; self.close();</script>";
 
@@ -78,7 +83,10 @@
                          'Cisco::Catalyst_3500XL' => 'Cisco Catalyst 3500XL',
                          'Cisco::Catalyst_3550' => 'Cisco Catalyst 3550',
                          'Cisco::Catalyst_3560' => 'Cisco Catalyst 3560',
+                         'Cisco::Catalyst_3750' => 'Cisco Catalyst 3750',
                          'Cisco::Controller_4400_4_2_130' => 'Cisco Controller 4400',
+                         'Cisco::ISR_1800' => 'Cisco Integrated Services Routers (ISR) 1800 Series',
+                         'Cisco::WiSM' => 'Cisco Wireless Services Module (WiSM)',
                          'Cisco::WLC_2106' => 'Cisco WLC 2106',
                          'Dell::PowerConnect3424' => 'Dell PowerConnect 3424',
                          'Dlink::DES_3526' => 'D-Link DES 3526',
