@@ -877,7 +877,10 @@ sub getMacAtIfIndex {
     my ( $this, $ifIndex ) = @_;
     my $logger = Log::Log4perl::get_logger( ref($this) );
     my $i      = 0;
+    my $start  = time;
     my @macArray;
+
+    # we try to get the MAC 30 times or for 2 minutes whichever comes first
     do {
         sleep(2) unless ( $i == 0 );
         $logger->debug( "attempt "
@@ -887,7 +890,17 @@ sub getMacAtIfIndex {
                 . " ifIndex $ifIndex" );
         @macArray = $this->_getMacAtIfIndex($ifIndex);
         $i++;
-    } while ( ( $i < 30 ) && ( scalar(@macArray) == 0 ) );
+    } while (($i < 30) && ((time-$start) < 120) && (scalar(@macArray) == 0));
+
+    if (scalar(@macArray) == 0) {
+        if ($i >= 30) {
+            $logger->warn("Tried to grab MAC address at ifIndex $ifIndex "
+                ."on switch ".$this->{_ip}." 30 times and failed");
+        } else {
+            $logger->warn("Tried to grab MAC address at ifIndex $ifIndex "
+                ."on switch ".$this->{_ip}." for 2 minutes and failed");
+        }
+    }
     return @macArray;
 }
 
@@ -1778,12 +1791,22 @@ sub isNewerVersionThan {
 
 sub generateFakeMac {
     my ( $this, $vlan, $ifIndex ) = @_;
-    return
-          "02:00:00:00:"
-        . ( ( $vlan eq 'VoIP' ) ? "01" : "00" ) . ":"
-        . ( ( length($ifIndex) == 1 )
-        ? "0" . substr( $ifIndex, -1, 1 )
-        : substr( $ifIndex, -2, 2 ) );
+    my $logger = Log::Log4perl::get_logger(ref($this));
+
+    # generating a fixed 6 digit string with ifIndex (zero filled)
+    my $zero_filled_ifIndex = sprintf('%06d', $ifIndex);
+    my $mac_suffix;
+    if ($zero_filled_ifIndex !~ /^\d{6}$/) {
+         $logger->warn("Unexpected ifIndex to generate a fake MAC for. "
+             . "This could cause port-security problems. ifIndex: $zero_filled_ifIndex");
+         $mac_suffix = "99:99:99";
+    } else {
+         $zero_filled_ifIndex =~ /(\d{2})(\d{2})(\d{2})/;
+         $mac_suffix = "$1:$2:$3";
+    }
+
+    # VoIP will be different than non-VoIP
+    return "02:00:" . ( ( $vlan eq 'VoIP' ) ? "01" : "00" ) . ":" . $mac_suffix;
 }
 
 sub isFakeMac {
@@ -1793,7 +1816,7 @@ sub isFakeMac {
 
 sub isFakeVoIPMac {
     my ( $this, $mac ) = @_;
-    return ( $mac =~ /^02:00:00:00:01/ );
+    return ( $mac =~ /^02:00:01/ );
 }
 
 sub getUpLinks {
@@ -1831,7 +1854,7 @@ Olivier Bilodeau <obilodeau@inverse.ca>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2006-2009 Inverse inc.
+Copyright (C) 2006-2010 Inverse inc.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
