@@ -23,38 +23,51 @@ use Log::Log4perl;
 use Log::Log4perl::Level;
 use Net::MAC;
 
-our (
-    $node_modify_sql,          $node_exist_sql,
-    $node_pid_sql,             $node_delete_sql,
-    $node_add_sql,             $node_regdate_sql,
-    $node_view_sql,            $node_count_all_sql,
-    $node_view_all_sql,        $node_view_with_fingerprint_sql,
-    $node_ungrace_sql,         $node_expire_window_sql,
-    $node_expire_deadline_sql, $node_expire_unreg_field_sql,
-    $node_expire_session_sql,  $node_expire_lastarp_sql,
-    $node_unregistered_sql,    $nodes_unregistered_sql,
-    $node_update_lastarp_sql,  $nodes_active_unregistered_sql,
-    $nodes_registered_sql,     $nodes_registered_not_violators_sql,
-    $nodes_active_sql,         $node_cleanup_sql,
-    $is_node_db_prepared
-);
+use constant NODE => 'node';
 
 BEGIN {
     use Exporter ();
     our ( @ISA, @EXPORT );
     @ISA = qw(Exporter);
-    @EXPORT
-        = qw(node_db_prepare node_exist node_pid node_delete node_add node_add_simple node_view node_count_all node_view_all node_view_with_fingerprint
-        node_modify node_register_auto node_register node_deregister nodes_maintenance node_unregistered
-        nodes_unregistered nodes_registered nodes_registered_not_violators nodes_active_unregistered
-        node_expire_lastarp node_cleanup node_update_lastarp node_mac_wakeup);
+    @EXPORT = qw(
+        node_db_prepare
+        $node_db_prepared
+
+        node_exist
+        node_pid
+        node_delete
+        node_add
+        node_add_simple
+        node_view
+        node_count_all
+        node_view_all
+        node_view_with_fingerprint
+        node_modify
+        node_register_auto
+        node_register
+        node_deregister
+        node_unregistered
+        nodes_maintenance
+        nodes_unregistered
+        nodes_registered
+        nodes_registered_not_violators
+        nodes_active_unregistered
+        node_expire_lastarp
+        node_cleanup
+        node_update_lastarp
+        node_mac_wakeup
+    );
 }
 
 use pf::config;
 use pf::db;
 use pf::util;
 
-$is_node_db_prepared = 0;
+# The next two variables and the _prepare sub are required for database handling magic (see pf::db)
+our $node_db_prepared = 0;
+# in this hash reference we hold the database statements. We pass it to the query handler and he will repopulate
+# the hash if required
+our $node_statements = {};
 
 =head1 SUBROUTINES
 
@@ -64,91 +77,78 @@ TODO: This list is incomlete
 
 =cut
 
-#node_db_prepare($dbh) if (!$thread);
-
 sub node_db_prepare {
-    my ($dbh) = @_;
-    db_connect($dbh);
     my $logger = Log::Log4perl::get_logger('pf::node');
     $logger->debug("Preparing pf::node database queries");
-    $node_exist_sql = $dbh->prepare(qq[ select mac from node where mac=? ]);
-    $node_pid_sql   = $dbh->prepare(
-        qq[ select count(*) from node where status='reg' and pid=? ]);
-    $node_add_sql
-        = $dbh->prepare(
-        qq[ insert into node(mac,pid,detect_date,regdate,unregdate,lastskip,status,user_agent,computername,notes,dhcp_fingerprint,last_arp,last_dhcp,switch,port,vlan) values(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?) ]
-        );
-    $node_delete_sql = $dbh->prepare(qq[ delete from node where mac=? ]);
-    $node_modify_sql
-        = $dbh->prepare(
-        qq[ update node set mac=?,pid=?,detect_date=?,regdate=?,unregdate=?,lastskip=?,status=?,user_agent=?,computername=?,notes=?,dhcp_fingerprint=?,last_arp=?,last_dhcp=?,switch=?,port=?,vlan=? where mac=? ]
-        );
-    $node_view_sql
-        = $dbh->prepare(
-        qq[ select node.mac,node.pid,node.detect_date,node.regdate,node.unregdate,node.lastskip,node.status,node.user_agent,node.computername,node.notes,node.last_arp,node.last_dhcp,node.dhcp_fingerprint,node.switch,node.port,node.vlan,ifnull(openviolations.nb,0) as nbopenviolations from node left join (select violation.mac,count(*) as nb from violation where status='open' group by violation.mac) as openviolations on node.mac=openviolations.mac where node.mac=? ]
-        );
-    $node_view_with_fingerprint_sql
-        = $dbh->prepare(
-        qq[ select mac,pid,detect_date,regdate,unregdate,lastskip,status,user_agent,computername,notes,last_arp,last_dhcp,ifnull(os_class.description, ' ') as dhcp_fingerprint,switch,port,vlan from node left join dhcp_fingerprint ON node.dhcp_fingerprint=dhcp_fingerprint.fingerprint LEFT JOIN os_mapping ON dhcp_fingerprint.os_id=os_mapping.os_type LEFT JOIN os_class ON os_mapping.os_class=os_class.class_id where mac=? ]
-        );
-    $node_view_all_sql
-        = "select node.mac,node.pid,node.detect_date,node.regdate,node.unregdate,node.lastskip,node.status,node.user_agent,node.computername,node.notes,node.last_arp,node.last_dhcp,node.dhcp_fingerprint,node.switch,node.port,node.vlan,ifnull(openviolations.nb,0) as nbopenviolations from node left join (select violation.mac,count(*) as nb from violation where status='open' group by violation.mac) as openviolations on node.mac=openviolations.mac";
-    $node_count_all_sql = "select count(*) as nb from node";
-    $node_ungrace_sql
-        = $dbh->prepare(
+
+    $node_statements->{'node_exist_sql'} = get_db_handle()->prepare(qq[ select mac from node where mac=? ]);
+
+    $node_statements->{'node_pid_sql'} = get_db_handle()->prepare(qq[ select count(*) from node where status='reg' and pid=? ]);
+
+    $node_statements->{'node_add_sql'} = get_db_handle()->prepare(
+        qq[ insert into node(mac,pid,detect_date,regdate,unregdate,lastskip,status,user_agent,computername,notes,dhcp_fingerprint,last_arp,last_dhcp,switch,port,vlan) values(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?) ]);
+
+    $node_statements->{'node_delete_sql'} = get_db_handle()->prepare(qq[ delete from node where mac=? ]);
+
+    $node_statements->{'node_modify_sql'} = get_db_handle()->prepare(
+        qq[ update node set mac=?,pid=?,detect_date=?,regdate=?,unregdate=?,lastskip=?,status=?,user_agent=?,computername=?,notes=?,dhcp_fingerprint=?,last_arp=?,last_dhcp=?,switch=?,port=?,vlan=? where mac=? ]);
+
+    $node_statements->{'node_view_sql'} = get_db_handle()->prepare(
+        qq[ select node.mac,node.pid,node.detect_date,node.regdate,node.unregdate,node.lastskip,node.status,node.user_agent,node.computername,node.notes,node.last_arp,node.last_dhcp,node.dhcp_fingerprint,node.switch,node.port,node.vlan,count(violation.mac) as nbopenviolations from node left join violation on node.mac=violation.mac and violation.status='open' where node.mac=? group by node.mac ]);
+
+    $node_statements->{'node_view_with_fingerprint_sql'} = get_db_handle()->prepare(
+        qq[ select mac,pid,detect_date,regdate,unregdate,lastskip,status,user_agent,computername,notes,last_arp,last_dhcp,ifnull(os_class.description, ' ') as dhcp_fingerprint,switch,port,vlan from node left join dhcp_fingerprint ON node.dhcp_fingerprint=dhcp_fingerprint.fingerprint LEFT JOIN os_mapping ON dhcp_fingerprint.os_id=os_mapping.os_type LEFT JOIN os_class ON os_mapping.os_class=os_class.class_id where mac=? ]);
+
+    # This guy here is special, have a look in node_view_all to see why
+    $node_statements->{'node_view_all_sql'}
+        = "select node.mac,node.pid,node.detect_date,node.regdate,node.unregdate,node.lastskip,node.status,node.user_agent,node.computername,node.notes,node.last_arp,node.last_dhcp,node.dhcp_fingerprint,node.switch,node.port,node.vlan,count(violation.mac) as nbopenviolations from node left join violation on node.mac=violation.mac and violation.status='open' group by node.mac";
+
+    # This guy here is special, have a look in node_count_all to see why
+    $node_statements->{'node_count_all_sql'} = "select count(*) as nb from node";
+
+    $node_statements->{'node_ungrace_sql'} = get_db_handle()->prepare(
         qq [ select mac from node where status="grace" and unix_timestamp(now())-unix_timestamp(lastskip) > ]
             . $Config{'registration'}{'skip_reminder'} );
-    $node_expire_unreg_field_sql
-        = $dbh->prepare(
-        qq [ select mac from node where status="reg" and unregdate != 0 and unregdate < now() ]
-        );
-    $node_expire_window_sql
-        = $dbh->prepare(
+
+    $node_statements->{'node_expire_unreg_field_sql'} = get_db_handle()->prepare(
+        qq [ select mac from node where status="reg" and unregdate != 0 and unregdate < now() ]);
+
+    $node_statements->{'node_expire_window_sql'} = get_db_handle()->prepare(
         qq [ select mac from node where status="reg" and unix_timestamp(regdate) + ]
             . $Config{'registration'}{'expire_window'}
             . qq[ < unix_timestamp(now()) ] );
-    $node_expire_deadline_sql
-        = $dbh->prepare(
+
+    $node_statements->{'node_expire_deadline_sql'} = get_db_handle()->prepare(
         qq [ select mac from node where status="reg" and regdate < ]
             . $Config{'registration'}{'expire_deadline'} );
-    $node_expire_session_sql
-        = $dbh->prepare(
-        qq [ update node n set n.status="unreg" where n.status="reg" and n.mac not in (select i.mac from iplog i where (i.end_time=0 or i.end_time > now())) and n.mac not in (select i.mac from iplog i where end_time!=0 and unix_timestamp(now())-unix_timestamp(i.end_time) < ]
-            . $Config{'registration'}{'expire_session'} );
-    $node_expire_lastarp_sql
-        = $dbh->prepare(
-        qq [ select mac from node where unix_timestamp(last_arp) < (unix_timestamp(now()) - ?) and last_arp!=0 ]
-        );
-    $node_unregistered_sql
-        = $dbh->prepare(
-        qq [ select mac,pid,detect_date,regdate,unregdate,lastskip,status,user_agent,computername,notes,last_arp,last_dhcp,dhcp_fingerprint,switch,port,vlan from node where status="unreg" and mac=? ]
-        );
-    $nodes_unregistered_sql
-        = $dbh->prepare(
-        qq [ select mac,pid,detect_date,regdate,unregdate,lastskip,status,user_agent,computername,notes,last_arp,last_dhcp,dhcp_fingerprint,switch,port,vlan from node where status="unreg" ]
-        );
-    $nodes_registered_sql
-        = $dbh->prepare(
-        qq [ select mac,pid,detect_date,regdate,unregdate,lastskip,status,user_agent,computername,notes,last_arp,last_dhcp,dhcp_fingerprint,switch,port,vlan from node where status="reg" ]
-        );
-    $nodes_registered_not_violators_sql
-        = $dbh->prepare(
-        qq [ select mac from node where status="reg" and mac not in (select mac from violation where status="open" group by mac) ]
-        );
-    $nodes_active_unregistered_sql
-        = $dbh->prepare(
-        qq [ select n.mac,n.pid,n.detect_date,n.regdate,n.unregdate,n.lastskip,n.status,n.user_agent,n.computername,n.notes,i.ip,i.start_time,i.end_time,n.last_arp from node n left join iplog i on n.mac=i.mac where n.status="unreg" and (i.end_time=0 or i.end_time > now()) ]
-        );
-    $nodes_active_sql
-        = $dbh->prepare(
-        qq [ select n.mac,n.pid,n.detect_date,n.regdate,n.unregdate,n.lastskip,n.status,n.user_agent,n.computername,n.notes,n.dhcp_fingerprint,i.ip,i.start_time,i.end_time,n.last_arp from node n, iplog i where n.mac=i.mac and (i.end_time=0 or i.end_time > now()) ]
-        );
-    $node_update_lastarp_sql
-        = $dbh->prepare(qq [ update node set last_arp=now() where mac=? ]);
 
-#$node_lookup_person_sql = $dbh->prepare( qq [ select mac,pid,description,user_agent,computername from node where pid=? ] );
-#$node_lookup_node_sql = $dbh->prepare( qq [ select mac,pid,description,user_agent,computername from node where mac=? ] );
-    $is_node_db_prepared = 1;
+    $node_statements->{'node_expire_session_sql'} = get_db_handle()->prepare(
+        qq [ update node n set n.status="unreg" where n.status="reg" and n.mac not in (select i.mac from iplog i where (i.end_time=0 or i.end_time > now())) and n.mac not in (select i.mac from iplog i where end_time!=0 and unix_timestamp(now())-unix_timestamp(i.end_time) < ] . $Config{'registration'}{'expire_session'} );
+
+    $node_statements->{'node_expire_lastarp_sql'} = get_db_handle()->prepare(
+        qq [ select mac from node where unix_timestamp(last_arp) < (unix_timestamp(now()) - ?) and last_arp!=0 ]);
+
+    $node_statements->{'node_unregistered_sql'} = get_db_handle()->prepare(
+        qq [ select mac,pid,detect_date,regdate,unregdate,lastskip,status,user_agent,computername,notes,last_arp,last_dhcp,dhcp_fingerprint,switch,port,vlan from node where status="unreg" and mac=? ]);
+
+    $node_statements->{'nodes_unregistered_sql'} = get_db_handle()->prepare(
+        qq [ select mac,pid,detect_date,regdate,unregdate,lastskip,status,user_agent,computername,notes,last_arp,last_dhcp,dhcp_fingerprint,switch,port,vlan from node where status="unreg" ]);
+
+    $node_statements->{'nodes_registered_sql'} = get_db_handle()->prepare(
+        qq [ select mac,pid,detect_date,regdate,unregdate,lastskip,status,user_agent,computername,notes,last_arp,last_dhcp,dhcp_fingerprint,switch,port,vlan from node where status="reg" ]);
+
+    $node_statements->{'nodes_registered_not_violators_sql'} = get_db_handle()->prepare(
+        qq [ select node.mac from node left join violation on node.mac=violation.mac and violation.status='open' where node.status='reg' group by node.mac having count(violation.mac)=0 ]);
+
+    $node_statements->{'nodes_active_unregistered_sql'} = get_db_handle()->prepare(
+        qq [ select n.mac,n.pid,n.detect_date,n.regdate,n.unregdate,n.lastskip,n.status,n.user_agent,n.computername,n.notes,i.ip,i.start_time,i.end_time,n.last_arp from node n left join iplog i on n.mac=i.mac where n.status="unreg" and (i.end_time=0 or i.end_time > now()) ]);
+
+    $node_statements->{'nodes_active_sql'} = get_db_handle()->prepare(
+        qq [ select n.mac,n.pid,n.detect_date,n.regdate,n.unregdate,n.lastskip,n.status,n.user_agent,n.computername,n.notes,n.dhcp_fingerprint,i.ip,i.start_time,i.end_time,n.last_arp from node n, iplog i where n.mac=i.mac and (i.end_time=0 or i.end_time > now()) ]);
+
+    $node_statements->{'node_update_lastarp_sql'} = get_db_handle()->prepare(qq [ update node set last_arp=now() where mac=? ]);
+
+    $node_db_prepared = 1;
     return 1;
 }
 
@@ -157,10 +157,9 @@ sub node_db_prepare {
 #
 sub node_exist {
     my ($mac) = @_;
-    node_db_prepare($dbh) if ( !$is_node_db_prepared );
-    $node_exist_sql->execute($mac) || return (0);
-    my ($val) = $node_exist_sql->fetchrow_array();
-    $node_exist_sql->finish();
+    my $query = db_query_execute(NODE, $node_statements, 'node_exist_sql', $mac) || return (0);
+    my ($val) = $query->fetchrow_array();
+    $query->finish();
     return ($val);
 }
 
@@ -169,10 +168,9 @@ sub node_exist {
 #
 sub node_pid {
     my ($pid) = @_;
-    node_db_prepare($dbh) if ( !$is_node_db_prepared );
-    $node_pid_sql->execute($pid) || return (0);
-    my ($count) = $node_pid_sql->fetchrow_array();
-    $node_pid_sql->finish();
+    my $query = db_query_execute(NODE, $node_statements, 'node_pid_sql', $pid) || return (0);
+    my ($count) = $query->fetchrow_array();
+    $query->finish();
     return ($count);
 }
 
@@ -181,7 +179,6 @@ sub node_pid {
 #
 sub node_delete {
     my ($mac) = @_;
-    node_db_prepare($dbh) if ( !$is_node_db_prepared );
     my $logger = Log::Log4perl::get_logger('pf::node');
     my $tmpMAC = Net::MAC->new( 'mac' => $mac );
     $mac = $tmpMAC->as_IEEE();
@@ -198,7 +195,7 @@ sub node_delete {
             return 0;
         }
     }
-    $node_delete_sql->execute($mac) || return (0);
+    db_query_execute(NODE, $node_statements, 'node_delete_sql', $mac) || return (0);
     $logger->info("node $mac deleted");
     return (1);
 }
@@ -208,7 +205,7 @@ sub node_delete {
 #
 sub node_add {
     my ( $mac, %data ) = @_;
-    node_db_prepare($dbh) if ( !$is_node_db_prepared );
+
     my $logger = Log::Log4perl::get_logger('pf::node');
     my $tmpMAC = Net::MAC->new( 'mac' => $mac );
     $mac = $tmpMAC->as_IEEE();
@@ -239,7 +236,7 @@ sub node_add {
         $data{regdate} = mysql_date();
     }
 
-    $node_add_sql->execute(
+    db_query_execute(NODE, $node_statements, 'node_add_sql',
         $mac,             $data{pid},              $data{detect_date},
         $data{regdate},   $data{unregdate},        $data{lastskip},
         $data{status},    $data{user_agent},       $data{computername},
@@ -277,20 +274,23 @@ sub node_add_simple {
 #
 sub node_view {
     my ($mac) = @_;
-    node_db_prepare($dbh) if ( !$is_node_db_prepared );
+
     my $tmpMAC = Net::MAC->new( 'mac' => $mac );
     $mac = $tmpMAC->as_IEEE();
-    $node_view_sql->execute($mac) || return (0);
-    my $ref = $node_view_sql->fetchrow_hashref();
+    my $query = db_query_execute(NODE, $node_statements, 'node_view_sql', $mac) || return (0);
+    my $ref = $query->fetchrow_hashref();
 
     # just get one row and finish
-    $node_view_sql->finish();
+    $query->finish();
     return ($ref);
 }
 
 sub node_count_all {
     my ( $id, %params ) = @_;
-    node_db_prepare($dbh) if ( !$is_node_db_prepared );
+    # Hack! we prepare the statement here so that $node_view_all_sql is pre-filled
+    node_db_prepare() if (!$node_db_prepared);
+    my $node_count_all_sql = $node_statements->{'node_count_all_sql'};
+
     if ( defined( $params{'where'} ) ) {
         if ( $params{'where'}{'type'} eq 'pid' ) {
             $node_count_all_sql
@@ -305,24 +305,33 @@ sub node_count_all {
             }
         }
     }
-    my $node_count_all_sth = $dbh->prepare($node_count_all_sql);
-    return db_data($node_count_all_sth);
+
+    # Hack! Because of the nature of the query built here (we cannot prepare it), we construct it as a string
+    # and pf::db will recognize it and prepare it as such
+    $node_statements->{'node_count_all_sql_custom'} = $node_count_all_sql;
+    return db_data(NODE, $node_statements, 'node_count_all_sql_custom');
 }
 
+=item * node_view_all
+
+=cut
 sub node_view_all {
-    node_db_prepare($dbh) if ( !$is_node_db_prepared );
     my ( $id, %params ) = @_;
+    # Hack! we prepare the statement here so that $node_view_all_sql is pre-filled
+    node_db_prepare() if (!$node_db_prepared);
+    my $node_view_all_sql = $node_statements->{'node_view_all_sql'};
+
     if ( defined( $params{'where'} ) ) {
         if ( $params{'where'}{'type'} eq 'pid' ) {
             $node_view_all_sql
-                .= " WHERE node.pid='" . $params{'where'}{'value'} . "'";
+                .= " HAVING node.pid='" . $params{'where'}{'value'} . "'";
         } elsif ( $params{'where'}{'type'} eq 'category' ) {
             require pf::nodecategory;
             my $cat      = $params{'where'}{'value'};
             my @catArray = pf::nodecategory::nodecategory_view($cat);
             if ( scalar(@catArray) == 1 ) {
                 my $sqlWhere = $catArray[0]->{'sql'};
-                $node_view_all_sql .= " WHERE " . $sqlWhere;
+                $node_view_all_sql .= " HAVING " . $sqlWhere;
             }
         }
     }
@@ -332,24 +341,26 @@ sub node_view_all {
     if ( defined( $params{'limit'} ) ) {
         $node_view_all_sql .= " " . $params{'limit'};
     }
-    my $node_view_all_sth = $dbh->prepare($node_view_all_sql);
-    return db_data($node_view_all_sth);
+
+    # Hack! Because of the nature of the query built here (we cannot prepare it), we construct it as a string
+    # and pf::db will recognize it and prepare it as such
+    $node_statements->{'node_view_all_sql_custom'} = $node_view_all_sql;
+    return db_data(NODE, $node_statements, 'node_view_all_sql_custom');
 }
 
 sub node_view_with_fingerprint {
     my ($mac) = @_;
-    node_db_prepare($dbh) if ( !$is_node_db_prepared );
-    $node_view_with_fingerprint_sql->execute($mac) || return (0);
-    my $ref = $node_view_with_fingerprint_sql->fetchrow_hashref();
+    my $query = db_query_execute(NODE, $node_statements, 'node_view_with_fingerprint_sql', $mac) || return (0);
+    my $ref = $query->fetchrow_hashref();
 
     # just get one row and finish
-    $node_view_with_fingerprint_sql->finish();
+    $query->finish();
     return ($ref);
 }
 
 sub node_modify {
     my ( $mac, %data ) = @_;
-    node_db_prepare($dbh) if ( !$is_node_db_prepared );
+
     my $tmpMAC = Net::MAC->new( 'mac' => $mac );
     $mac = $tmpMAC->as_IEEE();
     my $logger = Log::Log4perl::get_logger('pf::node');
@@ -416,7 +427,7 @@ sub node_modify {
         }
     }
 
-    $node_modify_sql->execute(
+    db_query_execute(NODE, $node_statements, 'node_modify_sql',
         $new_mac,                      $existing->{pid},
         $existing->{detect_date},      $existing->{regdate},
         $existing->{unregdate},        $existing->{lastskip},
@@ -561,13 +572,12 @@ sub node_deregister {
 
 sub nodes_maintenance {
     my $logger = Log::Log4perl::get_logger('pf::node');
-    node_db_prepare($dbh) if ( !$is_node_db_prepared );
 
     my $expire_mode = $Config{'registration'}{'expire_mode'};
     $logger->debug("nodes_maintenance called with expire_mode=$expire_mode");
 
-    $node_ungrace_sql->execute() || return (0);
-    while (my $row = $node_ungrace_sql->fetchrow_hashref()) {
+    my $ungrace_query = db_query_execute(NODE, $node_statements, 'node_ungrace_sql') || return (0);
+    while (my $row = $ungrace_query->fetchrow_hashref()) {
         my $currentMac = $row->{mac};
         `/usr/local/pf/bin/pfcmd manage deregister $currentMac`;
         $logger->info("modified $currentMac from status 'grace' to 'unreg'" );
@@ -576,8 +586,9 @@ sub nodes_maintenance {
     if ( isdisabled($expire_mode) ) {
         return (1);
     } else {
-        $node_expire_unreg_field_sql->execute() || return (0);
-        while (my $row = $node_expire_unreg_field_sql->fetchrow_hashref()) {
+        my $expire_unreg_query = db_query_execute(NODE, $node_statements, 'node_expire_unreg_field_sql')
+            || return (0);
+        while (my $row = $expire_unreg_query->fetchrow_hashref()) {
             my $currentMac = $row->{mac};
             `/usr/local/pf/bin/pfcmd manage deregister $currentMac`;
             $logger->info("modified $currentMac from status 'reg' to 'unreg' based on unregdate colum" );
@@ -586,19 +597,17 @@ sub nodes_maintenance {
         if (  ( lc($expire_mode) eq 'window' )
             && $Config{'registration'}{'expire_window'} > 0 )
         {
-            $node_expire_window_sql->execute() || return (0);
-            while (my $row = $node_expire_window_sql->fetchrow_hashref()) {
+            my $expire_window_query = db_query_execute(NODE, $node_statements, 'node_expire_window_sql') || return (0);
+            while (my $row = $expire_window_query->fetchrow_hashref()) {
                 my $currentMac = $row->{mac};
                 `/usr/local/pf/bin/pfcmd manage deregister $currentMac`;
                 $logger->info("modified $currentMac from status 'reg' to 'unreg' based on expiration window" );
             };
 
-        } elsif (  ( lc($expire_mode) eq 'deadline' )
-            && ( time - $Config{'registration'}{'expire_deadline'} > 0 ) 
-            &&  !( $Config{'network'}{'mode'} =~ /vlan/i ) )
-        {
-            $node_expire_deadline_sql->execute() || return (0);
-            while (my $row = $node_expire_deadline_sql->fetchrow_hashref()) {
+        } elsif ((lc($expire_mode) eq 'deadline' ) && ( time - $Config{'registration'}{'expire_deadline'} > 0 )) {
+            my $expire_deadline_query = db_query_execute(NODE, $node_statements, 'node_expire_deadline_sql') 
+                || return (0);
+            while (my $row = $expire_deadline_query->fetchrow_hashref()) {
                 my $currentMac = $row->{mac};
                 `/usr/local/pf/bin/pfcmd manage deregister $currentMac`;
                 $logger->info("modified $currentMac from status 'reg' to 'unreg' based on expiration deadline" );
@@ -607,8 +616,9 @@ sub nodes_maintenance {
         } elsif ( ( lc($expire_mode) eq 'session' ) 
             &&  !( $Config{'network'}{'mode'} =~ /vlan/i ) )
         {
-            $node_expire_session_sql->execute() || return (0);
-            my $rows = $node_expire_session_sql->rows;
+            my $expire_session_query = db_query_execute(NODE, $node_statements, 'node_expire_session_sql') 
+                || return (0);
+            my $rows = $expire_session_query->rows;
             $logger->log(
                 ( ( $rows > 0 ) ? $INFO : $DEBUG ),
                 "modified $rows nodes from status 'reg' to 'unreg' based on session expiration"
@@ -623,38 +633,31 @@ sub nodes_maintenance {
 sub node_unregistered {
     my ($mac) = @_;
 
-    node_db_prepare($dbh) if ( !$is_node_db_prepared );
-
-    $node_unregistered_sql->execute($mac) || return (0);
-    my $ref = $node_unregistered_sql->fetchrow_hashref();
-    $node_unregistered_sql->finish();
+    my $query = db_query_execute(NODE, $node_statements, 'node_unregistered_sql', $mac) || return (0);
+    my $ref = $query->fetchrow_hashref();
+    $query->finish();
     return ($ref);
 }
 
 sub nodes_unregistered {
-    node_db_prepare($dbh) if ( !$is_node_db_prepared );
-    return db_data($nodes_unregistered_sql);
+    return db_data(NODE, $node_statements, 'nodes_unregistered_sql');
 }
 
 sub nodes_registered {
-    node_db_prepare($dbh) if ( !$is_node_db_prepared );
-    return db_data($nodes_registered_sql);
+    return db_data(NODE, $node_statements, 'nodes_registered_sql');
 }
 
 sub nodes_registered_not_violators {
-    node_db_prepare($dbh) if ( !$is_node_db_prepared );
-    return db_data($nodes_registered_not_violators_sql);
+    return db_data(NODE, $node_statements, 'nodes_registered_not_violators_sql');
 }
 
 sub nodes_active_unregistered {
-    node_db_prepare($dbh) if ( !$is_node_db_prepared );
-    return db_data($nodes_active_unregistered_sql);
+    return db_data(NODE, $node_statements, 'nodes_active_unregistered_sql');
 }
 
 sub node_expire_lastarp {
     my ($time) = @_;
-    node_db_prepare($dbh) if ( !$is_node_db_prepared );
-    return db_data( $node_expire_lastarp_sql, $time );
+    return db_data(NODE, $node_statements, 'node_expire_lastarp_sql', $time);
 }
 
 sub node_cleanup {
@@ -671,8 +674,7 @@ sub node_cleanup {
 
 sub node_update_lastarp {
     my ($mac) = @_;
-    node_db_prepare($dbh) if ( !$is_node_db_prepared );
-    $node_update_lastarp_sql->execute($mac) || return (0);
+    db_query_execute(NODE, $node_statements, 'node_update_lastarp_sql', $mac) || return (0);
     return (1);
 }
 
@@ -697,16 +699,6 @@ sub node_mac_wakeup {
     pf::violation::violation_trigger( $mac, $dec_oui, "VENDORMAC" );
 }
 
-#sub node_lookup_person {
-#  my ($pid) = @_;
-#  return(db_data($node_lookup_person_sql,$pid));
-#}
-
-#sub node_lookup_node {
-#  my ($mac) = @_;
-#  return(db_data($node_lookup_node_sql,$mac));
-#}
-
 =back
 
 =head1 AUTHOR
@@ -727,7 +719,7 @@ Copyright (C) 2005 David LaPorte
 
 Copyright (C) 2005 Kevin Amorin
 
-Copyright (C) 2007-2009 Inverse inc.
+Copyright (C) 2007-2010 Inverse inc.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
